@@ -1,5 +1,6 @@
 """Class to make templates for digital effects tables for measurand"""
 
+import warnings
 from abc import ABC
 
 import numpy as np
@@ -374,6 +375,25 @@ class DigitalEffectsTableTemplates(ABC):
         :return: merged digital effects table
         :rtype: xarray.Dataset
         """
+        for var in ds.variables:
+            ds[var].encoding = ds_pre[var].encoding
+            err_corr_warn = False
+            for key in ds_pre[var].attrs.keys():
+                if "err_corr" in key:
+                    try:
+                        if ds[var].attrs[key] == ds_pre[var].attrs[key]:
+                            err_corr_warn = True
+                    except:
+                        err_corr_warn = True
+
+                else:
+                    ds[var].attrs[key] = ds_pre[var].attrs[key]
+            if err_corr_warn:
+                warnings.warn(
+                    "The returned dataset has a different error correlation parameterisation for variable %s than the provided ds_out_pre (possibly just the order changed)."
+                    % var
+                )
+
         if drop is not None:
             if drop in ds_pre.variables:
                 ds_pre = ds_pre.drop(drop)
@@ -382,6 +402,7 @@ class DigitalEffectsTableTemplates(ABC):
                 ds_pre = ds_pre.drop(var)
             except:
                 continue
+
         ds_out = xr.merge([ds, ds_pre])
         ds_out.attrs = ds_pre.attrs
 
