@@ -12,7 +12,9 @@ __status__ = "Development"
 
 
 class MeasurementFunctionUtils:
-    def __init__(self, xvariables, ydims, str_repeat_corr_dims, verbose, templ):
+    def __init__(
+        self, xvariables, ydims, str_repeat_corr_dims, verbose, templ, use_err_corr_dict
+    ):
         """
         Initialise MeasurementFunctionUtils
 
@@ -31,6 +33,7 @@ class MeasurementFunctionUtils:
         self.str_repeat_corr_dims = str_repeat_corr_dims
         self.templ = templ
         self.repeat_dims_form = "structured"
+        self.use_err_corr_dict = use_err_corr_dict
 
     def find_repeat_dim_corr(self, form, *args, store_unc_percent=False, ydims=None):
         """
@@ -342,7 +345,10 @@ class MeasurementFunctionUtils:
                 if hasattr(dataset, "variables"):
                     if var in dataset.variables:
                         if ydims is not None:
-                            if len(dataset[var].dims) < len(ydims):
+                            if np.all([dim in dataset[var].dims for dim in ydims]):
+                                inputs_unc[iv] = self.calculate_unc(form, dataset, var)
+
+                            else:
                                 inputs_unc[iv] = self.calculate_unc_missingdim(
                                     form,
                                     dataset,
@@ -351,8 +357,6 @@ class MeasurementFunctionUtils:
                                     sizes_dict=sizes_dict,
                                     ydims=ydims,
                                 )
-                            else:
-                                inputs_unc[iv] = self.calculate_unc(form, dataset, var)
 
             if np.count_nonzero(inputs_unc[iv]) == 0:
                 inputs_unc[iv] = None
@@ -377,14 +381,27 @@ class MeasurementFunctionUtils:
         :return: uncertainty values for the given variable in the given dataset. returns None if uncertainty component is not present in dataset.
         :rtype: np.ndarray
         """
+        data = None
         if form == "tot":
-            data = ds.unc[var].total_unc()
+            if self.use_err_corr_dict:
+                return [
+                    ds.unc[var][uvar].value.values
+                    for uvar in ds.unc._var_unc_var_names(var)
+                ]
+            else:
+                data = ds.unc[var].total_unc()
         elif form == "rand":
             data = ds.unc[var].random_unc()
         elif form == "syst":
             data = ds.unc[var].systematic_unc()
         elif form == "stru":
-            data = ds.unc[var].structured_unc()
+            if self.use_err_corr_dict:
+                return [
+                    ds.unc[var][uvar].value.values
+                    for uvar in ds.unc._var_unc_var_names(var, unc_type="structured")
+                ]
+            else:
+                data = ds.unc[var].structured_unc()
         else:
             try:
                 uvar = "%s_%s" % (form, var)
@@ -404,8 +421,6 @@ class MeasurementFunctionUtils:
                         return data.values / 100 * ds[var].values
                     else:
                         return data.values
-                else:
-                    data = None
 
         if data is not None:
             return data.values
@@ -535,9 +550,15 @@ class MeasurementFunctionUtils:
                 sli[i] = 0
         dsu = ds.unc[var][tuple(sli)]
         if form == "tot":
-            return dsu.total_err_corr_matrix().values
+            if self.use_err_corr_dict:
+                return [unc.err_corr_dict_numdim() for unc in dsu]
+            else:
+                return dsu.total_err_corr_matrix().values
         elif form == "stru":
-            return dsu.structured_err_corr_matrix().values
+            if self.use_err_corr_dict:
+                return [unc.err_corr_dict_numdim() for unc in dsu if unc.is_structured]
+            else:
+                return dsu.structured_err_corr_matrix().values
         elif form == "rand":
             return "rand"
         elif form == "syst":
@@ -559,7 +580,10 @@ class MeasurementFunctionUtils:
 
             try:
                 uvar = "%s_%s" % (form, var)
-                return dsu[uvar].err_corr_matrix().values
+                if self.use_err_corr_dict:
+                    return dsu[uvar].err_corr_dict_numdim()
+                else:
+                    return dsu[uvar].err_corr_matrix().values
             except:
                 keys = np.array(list(ds.keys()))
                 uvar_ids = [
@@ -567,7 +591,10 @@ class MeasurementFunctionUtils:
                 ]
                 uvar = keys[uvar_ids]
                 if len(uvar) > 0:
-                    return dsu[uvar[0]].err_corr_matrix().values
+                    if self.use_err_corr_dict:
+                        return dsu[uvar[0]].err_corr_dict_numdim()
+                    else:
+                        return dsu[uvar[0]].err_corr_matrix().values
                 else:
                     return None
 
