@@ -7,7 +7,6 @@ import unittest
 import comet_maths as cm
 import numpy as np
 import numpy.testing as npt
-
 from punpy.mc.mc_propagation import MCPropagation
 
 """___Authorship___"""
@@ -202,7 +201,7 @@ class TestMCPropagation(unittest.TestCase):
             return_samples=True,
             repeat_dims=1,
         )
-        npt.assert_allclose(ufb, yerr_corrb, atol=0.06)
+        npt.assert_allclose(ufb, yerr_corrb, atol=0.07)
 
     def test_propagate_random_1D_3var(self):
         prop = MCPropagation(20000, parallel_cores=1)
@@ -219,7 +218,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ufc, yerr_corrc, rtol=0.06)
 
     def test_propagate_random_3D_2out(self):
-        prop = MCPropagation(20000, parallel_cores=0)
+        prop = MCPropagation(20000, parallel_cores=0, verbose=True)
         ufd, ucorrd, corr_out = prop.propagate_random(
             functiond, xsd, xerrsd, return_corr=True, output_vars=2
         )
@@ -390,7 +389,7 @@ class TestMCPropagation(unittest.TestCase):
             function, xs, cov, return_corr=True, corr_between=np.ones((2, 2))
         )
         npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.06)
-        npt.assert_allclose(uf, yerr_corr, atol=0.1)
+        npt.assert_allclose(uf, yerr_corr, atol=0.15)
 
     def test_propagate_cov_2D(self):
         prop = MCPropagation(20000)
@@ -601,6 +600,21 @@ class TestMCPropagation(unittest.TestCase):
         xsd2 = [x1d, 1.0]
         xerrsd2 = [x1errd, 2.0]
 
+        prop = MCPropagation(20000, parallel_cores=1)
+
+        ufd, ucorrd, corr_out = prop.propagate_systematic(
+            functione,
+            xsd2,
+            xerrsd2,
+            return_corr=True,
+            corr_dims=0,
+            output_vars=2,
+        )
+        npt.assert_allclose(ucorrd[0], np.ones_like(ucorrd[0]), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+
+        prop = MCPropagation(20000, parallel_cores=3)
+
         ufd, ucorrd, corr_out = prop.propagate_systematic(
             functione,
             xsd2,
@@ -658,6 +672,58 @@ class TestMCPropagation(unittest.TestCase):
         )
         npt.assert_allclose(ucorrd[1], np.ones_like(ucorrd[1]), atol=0.01)
 
+    def test_separate_processing(self):
+        prop = MCPropagation(20000, parallel_cores=0)
+
+        corrd = [np.eye(len(xerrd.ravel())) for xerrd in xerrsd]
+
+        MC_x = prop.generate_MC_sample(xsd, xerrsd, corrd)
+        MC_y1 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000)
+        MC_y2 = prop.run_samples(functiond, MC_x, output_vars=2, start=10000, end=20000)
+        MC_y = prop.combine_samples([MC_y1, MC_y2])
+
+        ufd, ucorrd, corr_out = prop.process_samples(
+            MC_x, MC_y, return_corr=True, corr_dims=0, output_vars=2
+        )
+
+        npt.assert_allclose(ucorrd[0], np.eye(len(ucorrd[0])), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+        npt.assert_allclose(ufd[1], yerr_uncorrd[1], rtol=0.06)
+
+        prop = MCPropagation(20000, parallel_cores=0)
+
+        corrd = [np.eye(len(xerrd[:, 0, 0].ravel())) for xerrd in xerrsd]
+
+        MC_x = prop.generate_MC_sample(xsd, xerrsd, corr_x=["rand", "rand"])
+        MC_y1 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000)
+        MC_y2 = prop.run_samples(functiond, MC_x, output_vars=2, start=10000, end=20000)
+        MC_y = prop.combine_samples([MC_y1, MC_y2])
+
+        ufd, ucorrd, corr_out = prop.process_samples(
+            MC_x, MC_y, return_corr=True, corr_dims=0, output_vars=2
+        )
+
+        npt.assert_allclose(ucorrd[0], np.eye(len(ucorrd[0])), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+        npt.assert_allclose(ufd[1], yerr_uncorrd[1], rtol=0.06)
+
+        prop = MCPropagation(20000, parallel_cores=4)
+
+        corrd = [np.eye(len(xerrd[:, 0, 0].ravel())) for xerrd in xerrsd]
+
+        MC_x = prop.generate_MC_sample(xsd, xerrsd, corr_x=["rand", "rand"])
+        MC_y1 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000)
+        MC_y2 = prop.run_samples(functiond, MC_x, output_vars=2, start=10000, end=20000)
+        MC_y = prop.combine_samples([MC_y1, MC_y2])
+
+        ufd, ucorrd, corr_out = prop.process_samples(
+            MC_x, MC_y, return_corr=True, corr_dims=0, output_vars=2
+        )
+
+        npt.assert_allclose(ucorrd[0], np.eye(len(ucorrd[0])), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+        npt.assert_allclose(ufd[1], yerr_uncorrd[1], rtol=0.06)
+
     def test_perform_checks(self):
         prop = MCPropagation(20000)
         corrc = [
